pub enum AnyInstruction {
Real(Instruction),
Pseudo(PseudoInstruction),
}Variants
Real(Instruction)
Pseudo(PseudoInstruction)
Implementations
Sourceimpl AnyInstruction
impl AnyInstruction
Sourcepub const fn real(self) -> Option<Instruction>
pub const fn real(self) -> Option<Instruction>
Gets the inner value of Self::Real.
Sourcepub const fn pseudo(self) -> Option<PseudoInstruction>
pub const fn pseudo(self) -> Option<PseudoInstruction>
Gets the inner value of Self::Pseudo.
Sourcepub const fn expect_real(self) -> Instruction
pub const fn expect_real(self) -> Instruction
Same as Self::real but panics if wasn’t called on Self::Real.
Panics
If was called on something else other than Self::Real.
Sourcepub const fn expect_pseudo(self) -> PseudoInstruction
pub const fn expect_pseudo(self) -> PseudoInstruction
Same as Self::pseudo but panics if wasn’t called on Self::Pseudo.
Panics
If was called on something else other than Self::Pseudo.
Sourcepub fn is_block_push(&self) -> bool
pub fn is_block_push(&self) -> bool
Returns true if this is a block push pseudo instruction (SETUP_FINALLY, SETUP_CLEANUP, or SETUP_WITH).
Sourcepub fn is_pop_block(&self) -> bool
pub fn is_pop_block(&self) -> bool
Returns true if this is a POP_BLOCK pseudo instruction.